import bpy
from bpy.types import AddonPreferences, Operator
from render_polargrid.validation import (
    is_valid_email,
    is_valid_token,
    is_preferences_valid,
)
from render_polargrid.polargrid_client import Client
from render_polargrid.utils import get_settings
import render_polargrid.version_compatibility_utils as vcu


class PGPreferences(AddonPreferences):
    bl_idname = "render_polargrid"
    bl_label = "Authorization Token"

    backend = bpy.props.EnumProperty(
        name="Backend",
        description="Backend to use",
        items=(
            ('PRD', "Production", "Production infrastructure (farm.polargrid.space)"),
            ('TST', "Test", "Test infrastructure via firewall (farm.polargrid.space)"),
            ('DEV', "Development", "Development infrastructure (localhost:8000/8080)"),
        ),
        default='PRD',
        options=set(),
    )
    exec(vcu.convert_attribute_to_28("backend"))

    email = bpy.props.StringProperty(
        name="Email Address",
        description="Your Email address.\nYou will receive your authorization token on this Email",
        options={'TEXTEDIT_UPDATE'},
    )
    exec(vcu.convert_attribute_to_28("email"))

    token = bpy.props.StringProperty(
        name="Authorization Token",
        options={'TEXTEDIT_UPDATE'},
        subtype='PASSWORD',
    )
    exec(vcu.convert_attribute_to_28("token"))


    secret = bpy.props.StringProperty(
        subtype='PASSWORD',
        options=set(),
    )
    exec(vcu.convert_attribute_to_28("secret"))


    def draw(self, context):
        layout = self.layout

        if not is_preferences_valid(context):
            split = vcu.ui_split(ui_element=layout, factor=0.01)
            split.label(text="")
            col = split.column(align=True)
            col.label(text="After entering a valid email address you can request a token.")
            col.label(text="You will receive this token in an email.")
            col.label(text="When you received your token, add it here and press `Validate Token`.")
            col.label(text="This allows you to use the PolarGrid renderfarm.")

            layout.prop(self, "email", icon='NONE' if is_valid_email(self.email) else 'ERROR')
            layout.prop(self, "token", icon='NONE' if is_valid_token(self.token) else 'ERROR')

            row = layout.row()
            row.operator("polargrid.request_token")
            row.operator("polargrid.validate_token")

        else:
            row = layout.row()
            row.label(text="Email Address: "+self.email)
            row.label(text="Token has been validated")
            layout.operator("polargrid.request_token")

    @property
    def backend_url(self):
        return {
            'PRD': "https://farm.polargrid.space/",
            'TST': "https://farm.polargrid.space/",
            'DEV': "http://127.0.0.1:8080/",
        }[self.backend]


class PGRequestTokenOperator(Operator):
    bl_idname = "polargrid.request_token"
    bl_label = "Request / Renew Token"
    bl_description = "Receive a new token via email"

    @classmethod
    def poll(cls, context):
        settings = get_settings(context=context)
        return is_valid_email(settings.email)

    def execute(self, context):
        client = Client(context=context, reporter=self)
        settings = get_settings(context=context)
        client.request_token(
            email=settings.email,
            token=settings.token if is_valid_token(settings.token) else None,
            secret=settings.secret,
        )
        self.report({'INFO'}, "Please look in your inbox")
        settings.secret = ""
        return {'FINISHED'}


class PGValidateTokenOperator(Operator):
    bl_idname = "polargrid.validate_token"
    bl_label = "Validate Token"

    @classmethod
    def poll(cls, context):
        settings = get_settings(context=context)
        return is_valid_email(settings.email) and is_valid_token(settings.token)

    def execute(self, context):
        client = Client(context=context, reporter=self)
        settings = get_settings(context=context)
        secret = client.validate_token(email=settings.email, token=settings.token)
        if secret['secret'] is not None:
            settings.secret = secret['secret']
            self.report({'INFO'}, "Token is validated")
            bpy.ops.wm.save_userpref()

        else:
            settings.secret = ""
            self.report({'ERROR'}, "Token is invalid")

        return {'FINISHED'}
